@@ -27,9 +27,11 @@
 
 #if defined (HAVE_CAIRO_ATSUI)
 #  include "pangocairo-atsui.h"
-#elif defined (HAVE_CAIRO_WIN32)
+#endif
+#if defined (HAVE_CAIRO_WIN32)
 #  include "pangocairo-win32.h"
-#elif defined (HAVE_CAIRO_FREETYPE)
+#endif
+#if defined (HAVE_CAIRO_FREETYPE)
 #  include "pangocairo-fc.h"
 #endif
 
@@ -95,13 +97,64 @@ pango_cairo_font_map_new (void)
   return g_object_new (PANGO_TYPE_CAIRO_WIN32_FONT_MAP, NULL);
 #elif defined(HAVE_CAIRO_FREETYPE)
   return g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
+#else
+  g_assert_not_reached ();
+  return NULL;
 #endif
 }
 
+/**
+ * pango_cairo_font_map_new_for_font_type:
+ * @fonttype: desired #cairo_font_type_t
+ *
+ * Creates a new #PangoCairoFontMap object of the type suitable
+ * to be used with cairo font backend of type @fonttype.
+ *
+ * In most cases one should simply use @pango_cairo_font_map_new(),
+ * or in fact in most of those cases, just use
+ * @pango_cairo_font_map_get_default().
+ *
+ * Return value: the newly allocated #PangoFontMap of suitable type
+ *               which should be freed with g_object_unref(),
+ *               or %NULL if the requested cairo font backend is
+ *               not supported / compiled in.
+ *
+ * Since: 1.18
+ **/
+PangoFontMap *
+pango_cairo_font_map_new_for_font_type (cairo_font_type_t fonttype)
+{
+  /* Make sure that the type system is initialized */
+  g_type_init ();
+
+  switch (fonttype)
+  {
+#if defined(HAVE_CAIRO_ATSUI)
+    case CAIRO_FONT_TYPE_ATSUI:
+      return g_object_new (PANGO_TYPE_CAIRO_ATSUI_FONT_MAP, NULL);
+#endif
+#if defined(HAVE_CAIRO_WIN32)
+    case CAIRO_FONT_TYPE_WIN32:
+      return g_object_new (PANGO_TYPE_CAIRO_WIN32_FONT_MAP, NULL);
+#endif
+#if defined(HAVE_CAIRO_FREETYPE)
+    case CAIRO_FONT_TYPE_FT:
+      return g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
+#endif
+    default:
+      return NULL;
+  }
+}
+
 /**
  * pango_cairo_font_map_get_default:
  *
- * Gets a default font map to use with Cairo.
+ * Gets a default #PangoCairoFontMap to use with Cairo.
+ *
+ * Note that the type of the returned object will depend
+ * on the particular font backend Cairo was compiled to use;
+ * You generally should only use the #PangoFontMap and
+ * #PangoCairoFontMap interfaces on the returned object.
  *
  * Return value: the default Cairo fontmap for #Pango. This
  *  object is owned by Pango and must not be freed.
@@ -113,7 +166,7 @@ pango_cairo_font_map_get_default (void)
 {
   static PangoFontMap *default_font_map = NULL;
 
-  if (!default_font_map)
+  if (G_UNLIKELY (!default_font_map))
     default_font_map = pango_cairo_font_map_new ();
 
   return default_font_map;
@@ -183,17 +236,19 @@ pango_cairo_font_map_create_context (PangoCairoFontMap *fontmap)
 }
 
 /**
- * _pango_cairo_font_map_get_renderer:
- * @fontmap: a #PangoCairoFontmap
+ * pango_cairo_font_map_get_font_type:
+ * @fontmap: a #PangoCairoFontMap
  *
- * Gets the singleton #PangoCairoRenderer for this fontmap.
+ * Gets the type of Cairo font backend that @fontmap uses.  
  *
- * Return value: the singleton renderer
+ * Return value: the #cairo_font_type_t cairo font backend type
+ *
+ * Since: 1.18
  **/
-PangoRenderer *
-_pango_cairo_font_map_get_renderer (PangoCairoFontMap *fontmap)
+cairo_font_type_t
+pango_cairo_font_map_get_font_type (PangoCairoFontMap *fontmap)
 {
-  g_return_val_if_fail (PANGO_IS_CAIRO_FONT_MAP (fontmap), NULL);
+  g_return_val_if_fail (PANGO_IS_CAIRO_FONT_MAP (fontmap), CAIRO_FONT_TYPE_TOY);
 
-  return (* PANGO_CAIRO_FONT_MAP_GET_IFACE (fontmap)->get_renderer) (fontmap);
+  return (* PANGO_CAIRO_FONT_MAP_GET_IFACE (fontmap)->get_font_type) (fontmap);
 }
